
/*
  Data buffers.

  These buffers contain storage for data and a counter.
  The counter serves double duty as a buffer full flag. When it equals
  BUF_SIZE the foreground program will write it to main storage and reset the
  counter to zero.

  Number and size of buffers is set in buffer.h

  Don't get carried away and exceed the RAM space available.

*/
#include "LPC21xx.h"
#include <__armlib.h>
#include "fat.h"
#include "buffer.h"

#define stat0_on()    (IOCLR0 = 0x04)
#define stat0_off()   (IOSET0 = 0x04)
#define stat1_on()    (IOCLR0 = 0x800)
#define stat1_off()   (IOSET0 = 0x800)

struct data_buffer buffer[NUM_BUFFERS];

/*
  Interrupt time routine to output a single byte. Checks for a full
  buffer and switches to the next in line.

  If a buffer over run occurs (no space for data) an error condition is returned 
  and the data is not saved.
 */
int out_byte( char val )
{
  static int buf_index = 0;

  /* The current buffer should have one open space. In a buffer overrun
     condition it will not. Check for that and return an error.
  */
  if(buffer[buf_index].count == BUF_SIZE)
    return -1;

  /* Store the data.
   */
  buffer[buf_index].data[buffer[buf_index].count++] = val;

  /* Now check to see if the buffer is full. */

  if(buffer[buf_index].count == BUF_SIZE)
    {
      /* Step to the next buffer. Checking to see
	 if it is the last one.
      */
      buf_index++;
      if(buf_index >= NUM_BUFFERS)
	buf_index = 0;
    }
  return 0;
}

/*
  Mark all data buffers as empty.
 */
void init_buffers(void)
{
  int i;

  for(i = 0; i < NUM_BUFFERS; i++)
    buffer[i].count = 0;

  return;   
}

/*
  This is the foreground routine to handle data.

  Check the current buffer to see if it is full. If it is, issue
  a write command through the FAT file system code. Then mark it as empty.

  If the stop button is pressed, the file is closed and then everything
  stops.
 */
void handle_data(void)
{

  int buf_index;

  buf_index = 0;
  while(1)
    {
      if(buffer[buf_index].count == BUF_SIZE)
	{
	  /*
	    Blink the status lights in alternation. If the number of buffers
	    is odd, this will repeat the stat0 blink each time around.
	   */
	  (buf_index & 1) ? stat0_on() : stat1_on();
	  /*
	    The current action if the write fails is to close the file
            and stop.
	   */
	  if (fat_write(buffer[buf_index].data, BUF_SIZE/SECTOR_SIZE) ) //stop if we can't write
          {
              VICIntEnClr = 0xFFFFFFFF;  //clear all interrupts

              fat_close(buffer[0].data);
              blink_forever(2);
          }

	  (buf_index & 1) ? stat0_off() : stat1_off();
	  buffer[buf_index].count = 0;
	  if(++buf_index >= NUM_BUFFERS)
	    buf_index = 0;
        }

        
      //if the button has been pushed, log the rest of the file and quit
      if ((IOPIN0 & 0x00000008) == 0)    
        {
	  VICIntEnClr = 0xFFFFFFFF;  //clear all interrupts
#ifdef DEBUG
printf("Stopping\n");
#endif
	  /* There are several cases to handle.
	     1) buffers empty, nothing to do.
	     2) Current buffer partially full.
	     3) Several buffers full/partially full.

	     One loop handles them all.
	  */
	  while (buffer[buf_index].count != 0)
	    {
	      if( buffer[buf_index].count != BUF_SIZE )
		{
		  /* Pad buffer with SYNC. */
		  while(buffer[buf_index].count < BUF_SIZE)
                  {
		    out_byte(0x06);
                    out_byte(0xb9);
                  }
		}
	      fat_write(buffer[buf_index].data, BUF_SIZE/SECTOR_SIZE);

	      buffer[buf_index].count = 0;
	      if(++buf_index >= NUM_BUFFERS)
		buf_index = 0;
	    }

	  fat_close(buffer[0].data);
	  blink_forever(1);
        }
      /* This would be an excellent spot to put the processor in idle mode.
	 Since there is zero chance of a buffer spontaneously filling itself without
	 an interrupt from either the timer or UART, we can sleep. The interrupt will
	 start the ball rolling again.

	 This will decrease the average power used. According to the data sheet normal current
	 is about 40mA while idle current is about 10mA.
      */
      PCON |= 1;      /* Sleep until an interrupt occurs */
    }


}


/*
  Simple byte level file write routine just for writeDefaultData()

  Call with character to be written

  Uses buffer[0] since interrupt driven data isn't on.
 */

void sd_putc(char c)
{
  buffer[0].data[buffer[0].count++] = c;

  if(buffer[0].count == BUF_SIZE)
    {
      fat_write(buffer[0].data, BUF_SIZE/SECTOR_SIZE);
      buffer[0].count = 0;
    }
}


void sd_puts(char *p)
{
  while(*p != 0)
    sd_putc(*p++);
}

