/*
  Routines to access a FAT16 file system. THis was originally intended
  to be used in a rocket altimeter. Because the requirements of the
  altimeter are quite limited, this is not a general purpose package.

  The altimeter will first try to open a calibration file and read 
  calibration paramters from it. It will only need one sectors worth of data
  and will never open the file again.

  The altimeter will then create a file and begin writing to it. This will
  continue until end of flight when the file will be explicitly closed.
  As such, certain things are done and not done.

  When the file is created, a directory entry is created and it is
  initialized to point to the first free cluster. After this, it and the
  FAT chain are left  alone until the file is closed. The file is written
  assuming that all sectors after the initial one are free and available
  for this file. When the file is closed at end of flight, the FAT
  chain will be updated all at once and the correct file length written to
  the directory entry.

  This will of course leave the file in a tenuous state if for some reason
  it isn't closed properly. For example, it is not uncommon to power up the 
  altimeter during prep to verify it is functioning. The file will appear to
  be a single cluster long no matter what its length. If actual flight data
  is present, it can be recovered so long as no other data is written or
  files created. But it will require a special program to do it.

  The reason for handling things this way is that it simplifies the code
  a lot, reduces RAM requirements and speeds up writing of the data since
  getting a new cluster periodically is not required. It will also have
  the effect of decreasing number of erase/write cycles on the FAT sectors.

  When a sector buffer is required, the calling routine must provide a 
  pointer to it. There are no statically or dynamically allocated sector
  buffers in this code. This is in anticipation of using the FreeRTOS 
  system.


  This code is based on the FATLib.c code developed by Ivan Sham, William Hue,
  and Alex Jiang and published as part of an article for Circuit Cellar 
  magazine issue #

  Portions of that code may have been lifted wholesale and inserted here 
  although I am trying to write all my own stuff. That code is not exactly 
  well written. For example, the layering is done wierdly. There is a 
  function in the hardware layer whose sole purpose is to read sector
  zero and find the partition offset. This is a file system function
  and not a hardware function. Also, during initialization, sector zero is
  read more than once. Why? 

  I hope to do at least a little bit better. Portability will be an
  issue as I make assumptions about the sizes of ints (16 bits) and longs
  (32 bits) that may not be true on other machines.

  Information on the FAT file system is based on a white paper titled
  "Miscrosoft Extensible Firmware Initiative FAT32 File System Specification"
  version 1.03, Dec.6, 2000 by the Microsoft Corporation

  I tested this on a file built with mkfs and it appears to work correctly.
  Size of the compiled code is a little over 2K.

API:
int fat_init(unsigned char * buf)

   Call this routine with a pointer to a sector sized character buffer.
   The return value will be zero on success and nonzero for failure.

int readConfigurationData(char *filename, unsigned char *buf)

   Attempt to find the calibration file. If it exists the first sector is
   read into buf.
   The return value will be zero on success and nonzero for failure.

int fat_create(char *filename, unsigned char *buf)

   Attempt to create a file for writing. The filename _must_ be exactly as
   it appears in the directory. All uppercase, exactly 11 characters. buf
   is used as a scratch pad.
   The return value will be zero on success and nonzero for failure.

int fat_write(unsigned char * buf, int num)

   Write one or more sectors worth of data to the file. If more than
   one sector is to be written, the multi-sector write command is used.
   The return value will be zero on success and nonzero for failure.

int fat_close(buf)

   Close the file being written to. buf is used as a scratch pad for disk
   i/o.
   The return value will be zero on success and nonzero for failure.

2008
   I finally noticed that the SD cards have a multiple block write function
   and that it appears to be faster than block at a time. Therefore I am
   updating the code to use this feature.

   I have also decided to take steps to make use of the FAT chain more 
   robust. Currently I simply assume that all clusters after the first
   free cluster are also free. This may or may not be the case.

   To handle the times when it isn't the case, I do a little more work.
   This all happens up front so it will not effect the write speed. While
   fragmentation will occur in the file system it is more likely to happen
   at the beginning that at the end. So the method of finding the first
   cluster to write to is inverted. Start at the end of the FAT table and
   scan backwards for the first used cluster. This ensures a continuous
   section of free clusters. Free clusters from deleted files that have not 
   been returned to the free list, ie. changed to zero, are treated as used.
   This means that a file system that has been used for a while will have
   less capacity.

*/


#include <string.h>

#include "fat.h"
#include "sd.h"
#include "buffer.h"


/* This structure describes the mounted file system */
static struct  {
  int fstype;
  unsigned long SectorZero;   /* offset from absolute sector zero */
  unsigned long StartofData;  /* first sector with data */
  unsigned long RootDirectory; /* sector number of root directory */
  unsigned int SectorsPerFAT;  /* number of sectors in the FAT */
  unsigned int SectorsPerCluster; /* self explanatory */
  unsigned int ReservedSectors;
  unsigned long TotalSectors;
  unsigned int BytesPerSector;
  unsigned int NumFATS;
  unsigned int RootDirectorySectors;
  unsigned long countOfClusters;
  unsigned int lastFreeCluster;
} fstab;

/* This structure holds useful information on the file opened
   for writing.
*/

static struct  {
  unsigned long Directory;    // sector where directory entry sits
  unsigned int offset;        // offset of directory entry within sector
  unsigned int firstCluster;  // first cluster assigned to file
  unsigned int lastCluster;   // last cluster available to file
  unsigned long lastSector;    // last available sector for file
  unsigned long sector;       // next sector to write
  unsigned long numSectors;   // number of sectors written
} filetab;

/* Handle partition offset */

#define readSector(sector, buf)   readAbsoluteSector(sector+fstab.SectorZero, buf)
#define writeSector(sector, buf)  writeAbsoluteSector(sector+fstab.SectorZero, buf)

/*
  Read the file system information that describes its structure. Also
  decide what file system type it is.
*/

static int getFileSystemInfo(unsigned char * buf)
{
  unsigned int maxRootEntry;
  unsigned long datasectors;

  /* First verify that we have a FAT16 file system. The FATLib code
     does this by looking for the string "FAT16" at offset 54. 
     According to a Microsoft document, this is flat out wrong. It may
     work but it is wrong.
  */
  fstab.fstype = UNKNOWN;
  if(readSector( 0, buf))
    return -5;


  fstab.BytesPerSector = buf[11] | ((unsigned int)buf[12] << 8);
  fstab.SectorsPerCluster = buf[13];
  fstab.ReservedSectors = (unsigned int)buf[14] | ((unsigned int)buf[15] << 8);
  fstab.NumFATS = buf[16];
  maxRootEntry = (unsigned  int)buf[17] | ((unsigned int)buf[18] << 8);
  fstab.SectorsPerFAT = (unsigned int)buf[22] | ((unsigned int)buf[23] << 8);

  fstab.RootDirectory = fstab.ReservedSectors +
    fstab.SectorsPerFAT * fstab.NumFATS;

  fstab.RootDirectorySectors = ((unsigned long)maxRootEntry * 32 + 
				fstab.BytesPerSector - 1)/ fstab.BytesPerSector;
  fstab.StartofData = fstab.RootDirectory + fstab.RootDirectorySectors;

  if( fstab.SectorsPerFAT == 0 )
    return (fstab.fstype = FAT32);

  fstab.TotalSectors = buf[19] | ((unsigned int)buf[20] << 8);
      
  if( fstab.TotalSectors == 0 )     // see if we need to use the 32 bit version
    {
      fstab.TotalSectors = buf[32] |
	((unsigned long)buf[33] << 8) |
	((unsigned long)buf[34] << 16) |
	((unsigned long)buf[35] << 24);
    }

  datasectors = fstab.TotalSectors - (fstab.ReservedSectors +
				(fstab.NumFATS * fstab.SectorsPerFAT) +
				fstab.RootDirectory);
	
  

  fstab.countOfClusters = datasectors / fstab.SectorsPerCluster;


  if(fstab.countOfClusters < 4085)
    fstab.fstype = FAT12;
  else if(fstab.countOfClusters < 65525)
    fstab.fstype = FAT16;
  else fstab.fstype = FAT32;

  return( fstab.fstype );
}

/* Initialize FAT file system. returns 0 on success and -1 on error
 */


int fat_init(unsigned char *buf)
{
  unsigned long temp;

  // Make sure the open file data structure shows it isn't open
  // This will prevent any writes from working.
  filetab.sector = 0;

  /* First call the routine that initializes the SPI port and SD
     card. If that works, continue.
  */
  temp = SD_Init();
#ifdef DEBUG
  printf("SD_Init()= %d\n", temp);
#endif
  if( temp )
    return -1;

  /* 
     Check partition table for sector zero offset.
   */
  if(readAbsoluteSector(0, buf))
    return -2;

  temp = buf[457];
  temp <<= 8;
  temp |= buf[456];
  temp  <<= 8;
  temp |= buf[455];
  temp <<= 8;
  temp |= buf[454];

  fstab.SectorZero = temp;

  return getFileSystemInfo(buf);
    
}

void printfilesysteminfo()
{
  char buf[128];

  sprintf(buf, "Sector zero = %d (0x%x)\n", fstab.SectorZero, fstab.SectorZero);
  sd_puts(buf);
  sprintf(buf, "Start of Data = %d (0x%x)\n", fstab.StartofData, fstab.StartofData);
  sd_puts(buf);
  sprintf(buf, "RootDirectory = %d (0x%x)\n", fstab.RootDirectory, fstab.RootDirectory);
  sd_puts(buf);
  sprintf(buf, "Sectors per FAT = %d (0x%x)\n", fstab.SectorsPerFAT, fstab.SectorsPerFAT);
  sd_puts(buf);
  sprintf(buf, "Sectors per Cluster = %d (0x%x)\n", fstab.SectorsPerCluster, fstab.SectorsPerCluster);
  sd_puts(buf);
  sprintf(buf, "Reserved Sectors = %d (0x%x)\n", fstab.ReservedSectors, fstab.ReservedSectors);
  sd_puts(buf);
  sprintf(buf, "Total Sectors = %d (0x%x)\n", fstab.TotalSectors, fstab.TotalSectors);
  sd_puts(buf);
  sprintf(buf, "Bytes per Sector = %d (0x%x)\n", fstab.BytesPerSector, fstab.BytesPerSector);
  sd_puts(buf);
  sprintf(buf, "Number of FATS = %d (0x%x)\n", fstab.NumFATS, fstab.NumFATS);
  sd_puts(buf);
  sprintf(buf, "Root Directory sectors = %d (0x%x)\n", fstab.RootDirectorySectors, fstab.RootDirectorySectors);
  sd_puts(buf);
  sprintf(buf, "Count of Clusters = %d (0x%x)\n", fstab.countOfClusters, fstab.countOfClusters);
  sd_puts(buf);
}



/*
  Find the first unoccupied cluster in the FAT and return it. Also
  change that FAT entry to an end of file marker.

  This has been changed to scan from the end of the FAT for the first used
  cluster. This ensures that no used clusters get used. Cards with high
  levels of fragmentation will limit the maximum data file size to less
  than the card capacity.
*/

static unsigned long FirstEmptyCluster(unsigned char *buf)
{
  unsigned long sector, cluster;
  int i, j;
  unsigned char *bp;
  unsigned int firstfreeCluster;

  /* Scan the FAT from the last cluster towards the beginning */

  cluster = fstab.countOfClusters - 1;
  fstab.lastFreeCluster = 0;
  firstfreeCluster = 0;

  for( sector = fstab.ReservedSectors + (cluster >> 8); sector >= fstab.ReservedSectors; sector--)
    {
      if(readSector(sector, buf))
        return 0;
      bp = &buf[(cluster%256)*2];
      do {
	  /*
	    Since several table entries at the end may be unavailable,
	    scan for the last available cluster.
	   */
          if( *bp == 0 && *(bp+1) == 0)
          {
           if(fstab.lastFreeCluster == 0)
              fstab.lastFreeCluster = cluster;   
          }
          else           /* Found last used cluster */
	    {
	      /* First check for the case where this is the last
		 entry in a sector. In this case the first free cluster is in
		 the last sector we checked.
	      */

	      if((cluster+1)%256 == 0)
		{
		  sector++;
		  readSector(sector, buf);
		  bp = buf;
		}
	      else
		{
		  bp += 2;
		}
	      cluster++;
	      *bp++ = 0xff;
	      *bp = 0xff;

	      for( j = 0; j < fstab.NumFATS; j++)
		writeSector(sector + (j * fstab.SectorsPerFAT), buf);

	      return cluster;
	    }
            bp -= 2;
	} while( cluster-- % 256 );
    }
	  
  /* We failed to find a  free cluster */
  return 0;
}
	    

/* Look for the configuration file. Only files in the root directory are
   supported. Which simplifies things. 

   If successful, the first, and only, sector of the file is read into
   the buffer. No state information is saved so you can consider the file
   to be closed.

   returns -1 on failure and 0 on success

 
*/

int readConfigurationData(char *name, unsigned char *buf)
{
  int i, j;
  unsigned char *bp;
  unsigned int cluster;
  unsigned long sector;

  for(i = 0; i < fstab.RootDirectorySectors; i++)
    {
      readSector(fstab.RootDirectory + i, buf);
      bp = buf;      
      for( j = 0; j < 15; (bp += 32), j++)
	{
	  if(*bp == 0)
	    return -1;          /* End of directory */
	  if( *bp == 0xe5)
	    continue;
	  if( strncmp(name, bp, 11) == 0 )
	    {
	      /* We found our baby. Now grab the first cluster number */
	      cluster = (unsigned int)*(bp+26) | ((unsigned int)*(bp+27))<< 8;
	  
	      /* Convert the cluster number to a sector number */
	      sector = (cluster - 2) * fstab.SectorsPerCluster + fstab.StartofData;
	      /* And read that sector. */
	      readSector(sector, buf);
	      return 0;
	    }
	}
    }
  return -1;
}

/* Create a file to write data to. This does preserve state information.

   The filename must be in 8.3 format, without the dot and all uppercase.
   The altimeter code will use filenames of the form:

   FDATAXXX.DAT

   Where XXX is a number that gets incremented each time the altimeter
   tries to open a data file.

   Return values:

   -1           Failure, no need to try again. Ever.
   -2           Duplicate file name, try again
   0            Success, file is ready for data
*/

int fat_create( char *filename, unsigned char * buf, int mflag)
{
  int i, j;
  unsigned long temp;
  unsigned long cluster;
  unsigned char *bp;

  if( fstab.fstype != FAT16 )
    return -1;

    /* See if the same filename is already in use. Cheat by calling
      ReadConfiguration Data()
      */
  if(readConfigurationData(filename, buf)  == 0 )
    return -2;

  /* Find the first empty directory entry. 
  */
  for(i = 0; i < fstab.RootDirectorySectors; i++)
    {
      readSector(fstab.RootDirectory + i, buf);
      bp = buf;      
      for( j = 0; j < 15; (bp += 32), j++)
	{
	  if( *bp == 0 || *bp == 0xe5)
	    {
	      /* We have found an empty slot. Fill it in. */
	      /* First we get the first cluster since that will
		 trash the buffer.
	      */
	      if( (cluster = FirstEmptyCluster(buf)) == 0 )
		return -1;    /* Failure */

	      /* Now read the directory in again so we can update it. */
	      if(readSector(fstab.RootDirectory + i, buf))
                return -1;

	      strncpy(bp, filename, 11);
	      bp += 11;
	      *bp++ = 0;         // attribute bits
              *bp++ = 0;         // reserved
	      *bp++ = 0;         // create time, 10ms units
	      /* Phony up file creation date/time of noon, 1 Jan. 2006 */
	      *bp++ = 0;
	      *bp++ = 0x30;
	      *bp++ = 0x21;
	      *bp++ = 0x34;
	      /* and the last access date is the same */
	      *bp++ = 0x21;
	      *bp++ = 0x34;
	      /* Since this is not a FAT32 volume, these two are zero */
	      *bp++ = 0;
	      *bp++ = 0;
	      /* A phony write date/time as well */
	      *bp++ = 0;
	      *bp++ = 0x30;
	      *bp++ = 0x21;
	      *bp++ = 0x34;
	      /* now the first and so far only cluster */
	      *bp++ = cluster;     /* lower byte only */
	      *bp++ = (cluster >> 8);    /* Now the upper byte */
	      /* Now we add the file size. For now, it is one clusters 
		 worth.
	      */
	      temp = fstab.SectorsPerCluster * fstab.BytesPerSector;
	      *bp++ = temp;
	      temp >>= 8;
	      *bp++ = temp;
	      temp >>= 8;
	      *bp++ = temp;
	      temp >>= 8;
	      *bp++ = temp;
	      // Now write it back
	      if( writeSector(fstab.RootDirectory+i, buf) )
                return -1;

	      // Now store a few nice to know details
	      filetab.Directory = fstab.RootDirectory+i;
	      filetab.offset = j * 32;
	      filetab.sector = (cluster - 2) * fstab.SectorsPerCluster + fstab.StartofData;
	      filetab.numSectors = 0;
	      filetab.firstCluster = cluster;
	      filetab.lastCluster = fstab.lastFreeCluster;
	      filetab.lastSector = (filetab.lastCluster - 2) * fstab.SectorsPerCluster + fstab.StartofData;
	      return 0;
	    }
	}
    }
  return -1;         // Ran out of directory space somehow
}

/* Write one or more sectors for the open file.
*/

int fat_write(unsigned char * buf, int num)
{
  int i;

  if( filetab.sector == 0 )
    return -1;

    // Make sure we haven't run out of space. If we have, return
    // an error..
  if( filetab.sector + num - 1 >= filetab.lastSector )
      return -1;

  if(num > 1)
    {
      if(StartMultiBlockWrite(filetab.sector+fstab.SectorZero, num))
	return -1;

      while(num > 0)
	{
	  i = BlockWrite(buf);
          if(i)
          {
#ifdef DEBUG
printf("Error during multi block write. num = %d, 0x%x\n", num, i);
#endif
            StopMultiBlockWrite();
            return -1;
          }
          filetab.sector++;
	  buf += SECTOR_SIZE;
	  filetab.numSectors++;
	  num--;
        }
      StopMultiBlockWrite();

    }
  else
    {
      writeSector(filetab.sector++, buf);
      filetab.numSectors++;
    }
  return 0;
}

/*
  Close the file that we have been writing data to. First make sure that
  it  was opened and then do it.

  Only the first cluster has been allocated to this file although we
  wrote data to the disk as though every cluster after that had been
  allocated. Now we go back and fill in the FAT table. By doing it
  this way, the FAT table is only written to  twice. Once when we open
  the file and now. Since the target is a flash memory device, the number
  of erase/write cycles on any given location is limited. 

  We also need to update the file size in the directory entry.
*/

int fat_close(unsigned char * buf)
{
  unsigned int currentCluster, numClusters;
  unsigned long filesize;
  int j;
  unsigned long fatsector;
  unsigned char *bp;

  if(filetab.sector == 0)       // file not opened
    return -1;


  filesize = filetab.numSectors * fstab.BytesPerSector;

  /* Compute the number of clusters. This is tricky because we
     will almost certainly have a partially used cluster.
  */

  numClusters = (filetab.numSectors + (fstab.SectorsPerCluster - 1)) / 
  fstab.SectorsPerCluster;
  currentCluster = filetab.firstCluster;

  /* Now compute the sector number where our first cluster resides
     in the FAT.
  */
  fatsector = fstab.ReservedSectors + (filetab.firstCluster / 
				       (fstab.BytesPerSector >> 1));
#ifdef DEBUG
printf("Closing file, clusters %d to %d\n", currentCluster, currentCluster+numClusters);
#endif
 if(readSector(fatsector, buf))
   return -1;

  while(numClusters-- > 1)  // handles last cluster as special case
    {
      buf[(currentCluster & 0xff) << 1] = currentCluster + 1;
      buf[((currentCluster & 0xff) << 1)+1] = (currentCluster + 1) >> 8;

      if( (++currentCluster & 0xff) == 0)
	{
	  // Write FAT sector back to "disk"
	  for( j = 0; j < fstab.NumFATS; j++)
	    writeSector(fatsector + (j * fstab.SectorsPerFAT), buf);


	  // Read the next FAT sector
	  if(readSector(++fatsector, buf))
	    return -1;
	}
    }
  // Write the EOC mark in the last cluster

  buf[(currentCluster & 0xff) << 1] = 0xff;
  buf[((currentCluster & 0xff) << 1)+1] = 0xff;
  
  // Write FAT sector back to "disk"
  for( j = 0; j < fstab.NumFATS; j++)
    writeSector(fatsector + (j * fstab.SectorsPerFAT), buf);
 
  /* Read the directory entry. We stashed some information earlier to 
     make this easy.
  */

  if(readSector(filetab.Directory, buf) )
    return -1;

  /* If we had any idea of the real time, now would be a great time
     to update the timestamps on the file.
  */

#ifdef DEBUG
  printf("Directory sector %d, offset 0x%x, size = 0x%x\n", filetab.Directory, filetab.offset, filesize);
#endif

  bp = buf + filetab.offset + 28;    // point to file size

  *bp++ = filesize;
  filesize >>= 8;
  *bp++ = filesize;
  filesize >>= 8;
  *bp++ = filesize;
  filesize >>= 8;
  *bp   = filesize;

  writeSector(filetab.Directory, buf);
  filetab.sector = 0;               // Make sure that it looks like no file is opened

  return 0;
}




