/*Logomatic V02


 */

/*
  Extensive rewrite of several sections.

  Removed 10 copies of same code for ADC conversion and created a
  general purpose subroutine to do the task.

  Moved all of the interrupt time output functions to a single 
  subroutine that handles things a byte at a time.

  Replaced the three copies of the foreground buffering code with
  a single version. Also generalized the buffer code so that N
  buffers can be used. Limited only by available memory.

  Included a switch to idle mode in the main data loop. The power savings
  isn't as great as with some micro-controllers but it does save some. Dependent
  on interrupt frequency of course.

  Function should be the same with one exception. Mode 1 no longer pads each
  512 byte sector but records the data as a continuous stream.

  The code size is cut by about half. Not really significant given the 
  512K of flash memory. But my FORTH sensibilities are appeased. :-)

  All interrupt service routines moved to their own "ghetto" so that 
  optimization can be turned on for the rest of this file.

  
  dwschultz Aug 2008

*/


/**********************************************************
                  Header files
**********************************************************/
#include "LPC21xx.h"
#include <__armlib.h>
//#include <stdio.h>
#include <stdlib.h>
#include "fat.h"
#include "buffer.h"
#include "adc.h"
#include "irq.h"

#define IAP_LOCATION  0x7FFFFFF1
//#define FIQ_INTERRUPTS

  /* *********************************************************
  Function declarations
  ********************************************************* */
void __putchar(int ch);




void Initialize(void);
void feed(void);

void setup_uart0(int baud, char want_ints);
void setup_uart1(int baud, char want_ints);

void mode_0(void);
void mode_1(void);
void mode_2(void);
void mode_2_config(void);

void Log_init(void);
void test(void);

char getc(void);

void put_char(char);
void putc2(char);
char getc2(void);


extern struct data_buffer buffer[];

/**********************************************************
                  Global Variables
**********************************************************/
char port = 0;

char title[10];

//Default settings=================================================================================
static char mode = 0;    //uart mode, 0 = uart auto, 1 = uart trigger, 2 = adc
       char asc = 0;    //log in ascii mode
static int  baud = 19200;   //baud = 19200  
       int  freq = 100;  //100Hz default for ADC operations
       char trig = '$';  //standard start of nmea sentences
       int  frame = 100; //100 characters after trigger character IF it's in uart auto mode
char ad1_5 = 0;   //off
char ad1_4 = 0;   //off
char ad1_3 = 0;   //off
char ad0_3 = 0;   //off
char ad0_2 = 0;   //off
char ad0_1 = 0;   //off
char ad1_2 = 0;   //off
char ad0_4 = 0;   //off
char ad1_7 = 0;   //off
char ad1_6 = 0;   //off
int adc0clkdiv = 255;
int adc1clkdiv = 255;
int pllM = 4;

#define stat0_on()    (IOCLR0 = 0x04)
#define stat0_off()   (IOSET0 = 0x04)
#define stat1_on()    (IOCLR0 = 0x800)
#define stat1_off()   (IOSET0 = 0x800)

/*
  Short routine that does nothing but blink the status lights forever.

  Just blink the status lights forever. The argument is the frequency for the
  the cycle. Uses Timer0 interrupt.
 */

void blink_forever( unsigned long freq )
{
  unsigned long j;

  __ARMLIB_enableIRQ();
  T0TCR = 0x00000002;			//Reset counter and prescaler

  /* Timer0 interrupt is an IRQ interrupt */
  VICIntSelect &= ~0x00000010;
  /* Enable Timer0 interrupt */
  VICIntEnable |= 0x00000010;
  /* Use slot 2 for timer interrupt */
  VICVectCntl2 = 0x24;
  /* Set the address of ISR for slot 1 */
  VICVectAddr2 = (unsigned int)blinkISR;

  T0MCR = 0x00000003;			//On match reset the counter and generate an interrupt

  freq *= 4;
  T0MR0 = 14745600*pllM / freq;

  T0PR = 0x00000000;
	
  T0TCR = 0x00000001;			//enable timer

    while(1)
    {

     PCON |= 1;      /* Sleep until an interrupt occurs */
     stat0_on();
     PCON |= 1;      /* Sleep until an interrupt occurs */
     stat0_off();
     PCON |= 1;      /* Sleep until an interrupt occurs */
     stat1_on();
     PCON |= 1;      /* Sleep until an interrupt occurs */
     stat1_off();
  
     }

}


void __putchar(int ch)
{
  if (ch == '\n')
    put_char('\r');
  put_char(ch);
}

/**********************************************************
                       MAIN
**********************************************************/

int	main (void)
{
  __ARMLIB_enableFIQ();

  //iap_entry=(IAP)IAP_LOCATION;

  int i, j;
    
  Initialize();
  init_buffers();
  setup_uart0(19200, 0);


  i = fat_init(buffer[0].data);
#ifdef DEBUG
  printf("fat_init() returned %d\n", i);
#endif
  if(i != FAT16 )
    blink_forever(1);   // If we can't write data, there is no point in continuing

   

  for (i = 0; i < 5; i++)
    {
      stat0_on();
      for (j = 0; j < 5000000; j++);
      stat0_off();
      stat1_on();
      for (j = 0; j < 5000000; j++);
      stat1_off();
    }

  /* Read the configuration file. */

  Log_init();

#ifdef DEBUG
  printf("Configuration complete.\n");
#endif
  /* Open the output file. */

  for (i = 0; i < 255; i++)
    {
      sprintf(title, "LOG%04d TXT", i);

  if( (j = fat_create(title, buffer[0].data, 1)) == 0)
        break;
      if(j == -1)
      {
#ifdef DEBUG
      printf("Fatal error trying to create file. <%s>\n", title);
#endif
	blink_forever(2);
      }
      if(j == -2)
	continue;

    }
  /* Setup the selected mode */

    switch(mode)
      {
      case 0:
          setup_uart0(baud, 1);
          break;
      case 1:
          setup_uart0(baud, 2);
          break;
      case 2:
          mode_2_config();
          break;
      case 3:
          mode_3_config();
          break;
      default:
          setup_uart0(baud, 1);
      }

  /* Process and store the data. */
#ifdef DEBUG
  printf("Mode = %d, processing data\n", mode);
#endif

  handle_data();
}

/**********************************************************
                      Initialize
**********************************************************/

#define PLOCK 0x400

void Initialize(void)
{

  // Setting Multiplier and Divider values
  /* 
     Fosc = 14.7456MHZ
     M = 4
     P = 2
     CCLK = 4 * 14.7456MHz = 58.9824MHz
     Fcco = 2 * 2 * CCLK = 235.9296MHZ
  */
  PLLCFG=0x23;
  feed();
  
  // Enabling the PLL */	
  PLLCON=0x1;
  feed();
  
  // Wait for the PLL to lock to set frequency
  while(!(PLLSTAT & PLOCK)) ;
  
  // Connect the PLL as the clock source
  PLLCON=0x3;
  feed();
  
  // Enabling MAM and setting number of clocks used for Flash memory fetch (4 cclks in this case)
  MAMCR=0x2;
  MAMTIM=0x4;
  
  // Setting peripheral Clock (pclk) to System Clock (cclk)
  VPBDIV=0x01;
	
  PINSEL0 = 0xCF351505;     // enable uart0, enable uart1, enable SPI0 but with SSEL as IO, AD1.2,1.3,1.4 1.5 active
  PINSEL1 = 0x15441801;   //AD0.1,0.2,0.3,0.4,1.6,1.7 active
  // Don't have to set the direction of the UART pins after they're configured with PINSEL0
                                        
  IODIR0 |= 0x00000884;   //


  IOSET0 = 0x00000080;	//	

  //set up the SPI port
  S0SPCCR = 0x08;			// set up SPI clk to be pclk/8, this is the fastest we can go
  S0SPCR = 0x30;			//master, msb first, first clk edge, active high, no ints
	
}

/*
  Modifying the PLL settings requires that a special sequence be written to
  the PLLFEED register. This cannot be interrupted by interrupts.
*/
void feed(void)
{
  PLLFEED=0xAA;
  PLLFEED=0x55;
}







//Sends nate to the Transmit Register
void put_char(char c)
{
  if (port == 0)
    {
      while((U0LSR & 0x20) == 0);
      U0THR = c;
    }
  else if (port == 1)
    {
      while((U1LSR & 0x20) == 0);
      U1THR = c;
    }
}


//specific to UART0
char getc(void)
{
  while((U0LSR & 0x01) == 0);
  return (U0RBR);
}
   
char getc2(void)
{
  while((U1LSR & 0x01) == 0);
  return (U1RBR);
}


void putc2(char c)
{
  while((U1LSR & 0x20) == 0);
  U1THR = c;
}



void setup_uart0(int baud, char want_ints)
{

  int divisor;

  divisor = 14745600*pllM/(baud*16);

  //set up uart0 
  U0LCR 	= 0x83;			// 8 bits, no Parity, 1 Stop bit, DLAB = 1
	
  U0DLM = divisor >> 8;
  U0DLL = divisor;
	                    
  U0FCR	= 0x01;			// Not sure this is necessary, but the manual sez it is
  U0LCR 	= 0x03;        	// DLAB = 0

  if (want_ints == 1)
    {
      //__ARMLIB_enableFIQ();
      __ARMLIB_enableIRQ();

      //stat1_on();
      /* UART0 interrupt is an IRQ interrupt */
      //VICIntSelect |= 0x00000040;
      VICIntSelect &= ~0x00000040;
      /* Enable UART interrupt */
      VICIntEnable |= 0x00000040;
      /* Use slot 1 for UART0 interrupt */
      VICVectCntl1 = 0x26;
      /* Set the address of ISR for slot 1 */
      VICVectAddr1 = (unsigned int)UART0ISR;

            

      //VICIntSelect |= 0x00000080;		//bit 7 is for UART1, sets to FIQ (not IRQ)
      //VICIntEnable |= 0x00000080;	//enables UART1 interupt
      U0IER = 0x01;					//enable RX data available interupt, DLAB must be 0 to write
		
    }

  else if (want_ints == 2)
    {
      //__ARMLIB_enableFIQ();
      __ARMLIB_enableIRQ();

      //stat1_on();
      /* UART0 interrupt is an IRQ interrupt */
      //VICIntSelect |= 0x00000040;
      VICIntSelect &= ~0x00000040;
      /* Enable UART interrupt */
      VICIntEnable |= 0x00000040;
      /* Use slot 1 for UART0 interrupt */
      VICVectCntl2 = 0x26;
      /* Set the address of ISR for slot 1 */
      VICVectAddr2 = (unsigned int)UART0ISR_2;

            

      //VICIntSelect |= 0x00000080;		//bit 7 is for UART1, sets to FIQ (not IRQ)
      //VICIntEnable |= 0x00000080;	//enables UART1 interupt
      U0IER = 0x01;					//enable RX data available interupt, DLAB must be 0 to write
		
    }
	
  else if (want_ints == 0)
    {
      VICIntEnClr = 0x00000040;	//disables UART0 interupt
      U0IER = 0x00;					//disable RX data available interupt, DLAB must be 0 to write
    }
	
      
}


void setup_uart1(int baud, char want_ints)
{
  char q;
  int divisor;

  divisor = 14745600*pllM/(baud*16);

  //set up uart1
  U0LCR 	= 0x83;			// 8 bits, no Parity, 1 Stop bit, DLAB = 1
	
  U0DLM = divisor >> 8;
  U0DLL = divisor;


  //U1IER = 0x01;					//enable RX data available interupt, DLAB must be 0 to write               
  U1FCR	= 0x03;			// Not sure this is necessary, but the manual sez it is
  	
  U1LCR 	= 0x03;        	// DLAB = 0
	
  q = U1RBR;


  if (want_ints == 1)
    {
      //stat1_on();
      /* UART1 interrupt is an IRQ interrupt */
      VICIntSelect &= ~0x80;
      /* Enable UART interrupt */
      VICIntEnable = 0x80;
      /* Use slot 0 for UART1 interrupt */
      VICVectCntl0 = 0x27;
      /* Set the address of ISR for slot 0 */
      VICVectAddr0 = (unsigned int)UART1ISR;

      __ARMLIB_enableIRQ();

      //VICIntSelect |= 0x00000080;		//bit 7 is for UART1, sets to FIQ (not IRQ)
      //VICIntEnable |= 0x00000080;	//enables UART1 interupt
      U1IER = 0x01;					//enable RX data available interupt, DLAB must be 0 to write
		
    }
	
  else if (want_ints == 0)
    {
      VICIntEnClr = 0x00000080;	//disables UART1 interupt
      U1IER = 0x00;					//disable RX data available interupt, DLAB must be 0 to write
    }
	

}


/*
  If the configuration file is not found, this routine is called. It will 
  write a sample configuration file, the card specific data, and some
  file system info.
 */
char DefaultConfiguration[] = "MODE = 0\nASCII = N\nBaud = 19200\nFrequency = 100\n\
Trigger Character = $\nText Frame = 100\nAD1.5 = N\nAD1.4 = N\nAD1.3 = N\nAD0.3 = N\n\
AD0.2 = N\nAD0.1 = N\nAD1.2 = N\nAD0.4 = N\nAD1.7 = N\nAD1.6 = N\nSafty On = Y\n\
ADC0 CLK DIV = 255\nADC1 CLK DIV = 255\nM = 4\n";

void writeDefaultData()
{
  unsigned char csd[16];
  char *bufp;
  int i;
#ifdef DEBUG
printf("writing default config... ");
#endif
  if(fat_create("LOGCON  TXT", buffer[0].data, 0) != 0)
    blink_forever(1);

  sd_puts(DefaultConfiguration);
  sd_puts("\n\n\n\n");

  /* Read the Card specific data and print it */

  sd_puts("Card Specific Data = \n");

  readCSD(csd);
  for(i = 0; i < 16; i++)
    {
      sprintf(buffer[1].data, "%x ", csd[i]);
      sd_puts(buffer[1].data);

    }
  sd_puts("\n\n");
  printfilesysteminfo();
  sd_putc(0);
  fat_write(buffer[0].data, BUF_SIZE/SECTOR_SIZE);
  fat_close(buffer[0].data);
#ifdef DEBUG
printf("done\n");
#endif
}



void Log_init(void)
{
  char *p, *s;
  int x, mark, index;
  char temp, temp2, safety;
  int stringSize;

  temp2 = mark = index = 0;

  if(readConfigurationData("LOGCON  TXT", buffer[0].data) != 0)
    {
      writeDefaultData();
      blink_forever(1);

    }

  /* Slightly reorganized scanner for the configuration file. It still has
     no flexibility.
   */
  p = buffer[0].data;  
  while(*p)
    {
      /* Look for the end of the line */
      if( *p == '\n' || *p == '\r')
	{
	  index++;
	  switch(index)
	    {
	    case 1:
	      mode = *(p-1) - '0';   //0 = auto uart, 1 = trigger uart, 2 = adc
	      break;
	    case 2:
	      asc = *(p-1) == 'Y' ? 1 : 0;
	      break;
	    case 3:
              s = p-1;
	      while(isdigit(*s))
		--s;
	      baud = atoi(s);
	      break;
	    case 4:
	      s = p-1;
	      while(isdigit(*s))
		--s;
	      freq = atoi(s);
	      break;
	    case 5:
	      trig = *(p-1);
	      break;
	    case 6:
	      s = p-1;
	      while(isdigit(*s))
		--s;
	      frame = atoi(s);
	      if(frame > 510)
		frame = 510;
	      break;
	    case 7:
	      if(*(p-1) == 'Y')
		{
		  ad1_5++;
		  temp2++;
		}
	      break;
	    case 8:
	      if(*(p-1) == 'Y')
		{
		  ad1_4++;
		  temp2++;
		}
	      break;
	    case 9:
	      if(*(p-1) == 'Y')
		{
		  ad1_3++;
		  temp2++;
		}
	      break;
	    case 10:
	      if(*(p-1) == 'Y')
		{
		  ad0_3++;
		  temp2++;
		}
	      break;
	    case 11:
	      if(*(p-1) == 'Y')
		{
		  ad0_2++;
		  temp2++;
		}
	      break;
	    case 12:
	      if(*(p-1) == 'Y')
		{
		  ad0_1++;
		  temp2++;
		}
	      break;
	    case 13:
	      if(*(p-1) == 'Y')
		{
		  ad1_2++;
		  temp2++;
		}
	      break;
	    case 14:
	      if(*(p-1) == 'Y')
		{
		  ad0_4++;
		  temp2++;
		}
	      break;
	    case 15:
	      if(*(p-1) == 'Y')
		{
		  ad1_7++;
		  temp2++;
		}
	      break;
	    case 16:
	      if(*(p-1) == 'Y')
		{
		  ad1_6++;
		  temp2++;
		}
	      break;
	    case 17:
	      safety = *(p-1);
	      break;
	    case 18:
	      s = p-1;
	      while(isdigit(*s))
		--s;
	      adc0clkdiv = atoi(s) - 1;
	      break;
	    case 19:
	      s = p-1;
	      while(isdigit(*s))
		--s;
	      adc1clkdiv = atoi(s) - 1;
	      break;
	    case 20:
	      s = p-1;
	      while(isdigit(*s))
		--s;
	      pllM = atoi(s);
	      break;

	    }
	  /* Some systems use just a LF to terminate a line and others use
	     a CR-LF pair. Skip over the CR..
	  */
	}
      if(*p++ == '\r')
	 p++;
     }

  if( adc0clkdiv < 13 )
    adc0clkdiv = 13;
  if( adc0clkdiv > 255 )
    adc0clkdiv = 255;
  if( adc1clkdiv < 13 )
    adc1clkdiv = 13;
  if( adc1clkdiv > 255 )
    adc1clkdiv = 255;

  if(pllM < 1 || pllM > 4)
  pllM = 4;

  if (safety == 'Y')    //make sure the frequency is within limits for ADC mode if safety is on
    {
      if(temp2 == 0 )
	freq = 100;
      else if(freq > 1500/temp2)
	freq = 1500/temp2;
    }

  if (safety == 'T') test();

}

/*
  Setup the ADC for burst mode.

  I would prefer to use the faster FIQ interrupt but I haven't figured out how
  to make those work yet.
 */
char pllcfgbit[] = {0, 0x60, 0x41, 0x22 };
char mamcfgbit[] = {0, 1, 2, 3 };

mode_3_config()
{
  int i;
  int adc0mask, adc1mask;
  int adc0cnt, adc1cnt;

  /* There is an option to run the CPU clock at a slower speed. M is already
     set to 4 and the only other options are 1, 2, and 3:

     M      CPU Clock
     1      14.7456 MHz
     2      29.4912 MHz
     3      44.2368 MHz

     The P value is also set appropriately.
  */

  if(pllM >= 1 && pllM <= 3)
    {
      PLLCON = 1;                  // Disconnect but leave the PLL running
      feed();

      PLLCFG = pllcfgbit[pllM];
      feed();
      while(!(PLLSTAT & PLOCK))   // Wait for PLL to stabilize
        ;
      PLLCON = 0x03;           // Connect PLL as clock source
      feed();

      /* Also adjust the MAM settings */
      MAMCR = 0;               // turn it off
      MAMTIM = mamcfgbit[pllM];
      MAMCR = 2;               // MAM fully enabled
      setup_uart0(19200, 0);   // since the clock changed...
    }


  /* Setup the interrupt handler */

  __ARMLIB_enableIRQ();

  VICVectCntl2 = 0x32;
  VICVectAddr2 = (int)&adc_irq0_handler;

  VICVectCntl3 = 0x35;
  VICVectAddr3 = (int)&adc_irq1_handler;

  VICIntSelect &= ~0x240000;         // AD0 and AD1 are IRQ interrupts
  VICIntEnable |= 0x240000;         // Enable AD0 and AD1 interrupt

  /*
   * First build the values used to enable ADC channels. One bit set for
   * each channel used.
   */
  adc0mask = adc0cnt = 0;
  if(ad0_1)
    {
      adc0cnt++;
      adc0mask |= 0x02;
    }
  if(ad0_2)
    {
      adc0cnt++;
      adc0mask |= 0x04;
    }
  if(ad0_3)
    {
      adc0cnt++;
      adc0mask |= 0x08;
    }
  if(ad0_4)
    {
      adc0cnt++;
      adc0mask |= 0x10;
    }

  adc1mask = adc1cnt = 0;
  if(ad1_2)
    {
      adc1cnt++;
      adc1mask |= 0x04;
    }
  if(ad1_3)
    {
      adc1cnt++;
      adc1mask |= 0x08;
    }
  if(ad1_4)
    {
      adc1cnt++;
      adc1mask |= 0x10;
    }
  if(ad1_5)
    {
      adc1cnt++;
      adc1mask |= 0x20;
    }
  if(ad1_6)
    {
      adc1cnt++;
      adc1mask |= 0x40;
    }
  if(ad1_7)
    {
      adc1cnt++;
      adc1mask |= 0x80;
    }
  


  /* Output the 16 bytes of header data */

  out_byte(3);             // mode = 3
  out_byte(adc0cnt);
  out_byte(adc1cnt);
  out_byte(adc0clkdiv);
  out_byte(adc1clkdiv);
  out_byte(pllM);

  for(i = 6; i < 14; i++ )
    out_byte(0);

  out_byte(0x06);         // end with SYNC pattern
  out_byte(0xb9);

  if(adc0cnt)
    AD0CR = adc0mask | (adc0clkdiv << 8) | 0x200000;
  if(adc1cnt)
    AD1CR = adc1mask | (adc1clkdiv << 8) | 0x200000;
  ADGSR |= 0x10000;                   // Use global start register to start both ADC's

}



void mode_2_config(void)
{
  __ARMLIB_enableIRQ();

  //stat1_on();
  /* Timer0 interrupt is an IRQ interrupt */
  VICIntSelect &= ~0x00000010;
  /* Enable Timer0 interrupt */
  VICIntEnable |= 0x00000010;
  /* Use slot 2 for UART0 interrupt */
  VICVectCntl2 = 0x24;
  /* Set the address of ISR for slot 1 */
  VICVectAddr2 = (unsigned int)MODE2ISR;

  T0TCR = 0x00000002;			//Reset counter and prescaler
  T0MCR = 0x00000003;			//On match reset the counter and generate an interrupt

  T0MR0 = 58982400 / freq;

  T0PR = 0x00000000;
	
  T0TCR = 0x00000001;			//enable timer
#ifdef DEBUG
printf("Timer freq = %d\n", freq);
#endif
}

void test(void)
{

  int temp, temp2;
  int j;
  char a;

  printf("Logomatic Test Code:\r\n");
  printf("Hit any key to test ADC lines, hit the stop button to terminate the test.\r\n\n");

  a = getc();
    
    
  while ((IOPIN0 & 0x00000008) == 0x00000008)
    {
      //Get AD1.5=======================================================================
      AD1CR = 0x0020FF20;	//AD1.5
      AD1CR |= 0x01000000;//start conversion
        
      while ((temp & 0x80000000) == 0)
        {
	  temp = AD1DR;
        }
        
        
      temp &= 0x0000FFC0;
      temp2 = temp / 0x00000040;
        
      AD1CR = 0x00000000;
        
      printf("%d",temp2);
      put_char(9);
                
        
      //Get AD1.4=======================================================================
      AD1CR = 0x0020FF10;	//AD1.4
      AD1CR |= 0x01000000;//start conversion
        
      while ((temp & 0x80000000) == 0)
        {
	  temp = AD1DR;
        }
        
        
      temp &= 0x0000FFC0;
      temp2 = temp / 0x00000040;
        
      AD1CR = 0x00000000;
        
      printf("%d",temp2);
      put_char(9);
            
      //Get AD1.3=======================================================================
      AD1CR = 0x0020FF08;	//AD1.3
      AD1CR |= 0x01000000;//start conversion
        
      while ((temp & 0x80000000) == 0)
        {
	  temp = AD1DR;
        }
        
        
      temp &= 0x0000FFC0;
      temp2 = temp / 0x00000040;
        
      AD1CR = 0x00000000;
                
      printf("%d",temp2);
      put_char(9);
            
      //Get AD0.3=======================================================================
      AD0CR = 0x0020FF08;	//AD0.3
      AD0CR |= 0x01000000;//start conversion
        
      while ((temp & 0x80000000) == 0)
        {
	  temp = AD0DR;
        }
        
        
      temp &= 0x0000FFC0;
      temp2 = temp / 0x00000040;
        
      AD0CR = 0x00000000;
                
      printf("%d",temp2);        
      put_char(9);
            
      //Get AD0.2=======================================================================
      AD0CR = 0x0020FF04;	//AD0.2
      AD0CR |= 0x01000000;//start conversion
        
      while ((temp & 0x80000000) == 0)
        {
	  temp = AD0DR;
        }
        
      temp &= 0x0000FFC0;
      temp2 = temp / 0x00000040;
        
      AD0CR = 0x00000000;
                
      printf("%d",temp2);  
      put_char(9);
              
      //Get AD0.1=======================================================================
      AD0CR = 0x0020FF02;	//AD0.1
      AD0CR |= 0x01000000;//start conversion
        
      while ((temp & 0x80000000) == 0)
        {
	  temp = AD0DR;
        }
        
      temp &= 0x0000FFC0;
      temp2 = temp / 0x00000040;
        
      AD0CR = 0x00000000;
    
      printf("%d",temp2);
      put_char(9);
                
            
      //Get AD1.2========================================================================
      AD1CR = 0x0020FF04;	//AD0.0, P0.27
      AD1CR |= 0x01000000;	//start conversion
        
      while ((temp & 0x80000000) == 0)
        {
	  temp = AD1DR;
        }
        
      temp &= 0x0000FFC0;
      temp2 = temp / 0x00000040;
        
      AD1CR = 0x00000000;
                
      printf("%d",temp2);        
      put_char(9);
            
      //Get AD0.4=======================================================================
      AD0CR = 0x0020FF10;	//AD0.4
      AD0CR |= 0x01000000;//start conversion
        
      while ((temp & 0x80000000) == 0)
        {
	  temp = AD0DR;
        }
        
      temp &= 0x0000FFC0;
      temp2 = temp / 0x00000040;
        
      AD0CR = 0x00000000;
                
      printf("%d",temp2);        
      put_char(9);
            
      //Get AD1.7=======================================================================
      AD1CR = 0x0020FF80;	//AD1.7
      AD1CR |= 0x01000000;//start conversion
        
      while ((temp & 0x80000000) == 0)
        {
	  temp = AD1DR;
        }
        
      temp &= 0x0000FFC0;
      temp2 = temp / 0x00000040;
        
      AD1CR = 0x00000000;
    
      printf("%d",temp2);
      put_char(9);
                
        
            
      //Get AD1.6=======================================================================
      AD1CR = 0x0020FF40;	//AD1.6
      AD1CR |= 0x01000000;//start conversion
        
      while ((temp & 0x80000000) == 0)
        {
	  temp = AD1DR;
        }
        
      temp &= 0x0000FFC0;
      temp2 = temp / 0x00000040;
        
      AD1CR = 0x00000000;
    
      printf("%d\r\n",temp2);
       
      for (j = 0; j < 800000; j++);
      put_char(10);
      put_char(13);
    }


  printf("\r\nTest Complete.\r\n");

  blink_forever(4);


}


