/*
  sd.c

  Routines to access SD (and MMC) memory cards.

  Based on V2.0 SD card specification.

  SDHC cards are not supported. Minor changes to the 
  initialization code are needed and something must be done
  about addressing. SD cards use a byte level address while
  SDHC use a sector level address. (With fixed sector size of 512 bytes)

 */

#include "LPC21xx.h"
#include "spi.h"


#define NONE  0                   // Card types
#define MMC   1
#define SD    2
#define SDHC  3

static int CardType = NONE;


#define CMD0     0            // Command Card into Idle state
#define CMD1     1
#define CMD8     8            // Send Interface Condition Command
#define CMD9     9            // Read CSD registers
#define CMD13    13           // Send Status
#define CMD16    16           // Set block length
#define CMD17    17           // Read block
#define CMD24    24           // Write block
#define CMD25    25           // Write multiple blocks
#define CMD55    55           // SIgnal next command is application specific
#define CMD58    58
#define ACMD22   (0x80 | 22)  // Retrieve number of well written blocks after multi block write
#define ACMD23   (0x80 | 23)  // Set write block erase count
#define ACMD41   (0x80 | 41)

#define SECTOR_SIZE 512

/* Compute CRC7 on data one byte at a time. 
 */
unsigned char crc7( unsigned char crc, unsigned char data)
{
  int i;

  for(i = 0; i < 8; i++)
    {
      crc <<= 1;
      if( (crc ^ data) & 0x80 )
	crc ^= 0x9;
      data <<= 1;
    }
  return crc;
}

/*
  Since the ARM pins can be used for only one function it is impossible
  to test the state of the data in pin by just reading the port pin
  register. Or so the documentation says. The only alternative left
  is to send a dummy byte and see what data is read.

  A timeout would probably be a good idea.

*/

void busyWait()
{
  while(TxRxbyte(0xff) != 0xff)
    ;
}

/* Special version of send command routine since this command has 
   no argument and a R2 format response.
*/
static int SendStatus( )
{
  int i, response;

  busyWait();

  TxRxbyte(CMD13 | 0x40 );

  /* Now wait for the response */

  for(i = 0; i < 64; i++ )
    {
      if( (response = TxRxbyte(0xff)) != 0xff )
	break;
    }
  response << 8;
  response |= TxRxbyte(0xff);
  return response;
}


/*
  Send command and arguement to SD card. If cflag is non-zero then
  a CRC7 will be computed and sent. Otherwise a dummy value is sent
  in place of the CRC7.
 */

static int SendCmd( unsigned char command, unsigned long arg, int cflag)
{
  char c, crc;
  int i, response;

  if( command & 0x80 )
  {
    response = SendCmd(CMD55, 0, 0);
    if(response & 0x7e)    // ignore idle bit
      return response;
    command &= 0x7f;
  }

  busyWait();

  if(cflag)
    {
      c = command | 0x40;
      crc = crc7(0, c);
      TxRxbyte( c );

      c = arg >> 24;
      crc = crc7(crc, c);
      TxRxbyte( c );

      c = arg >> 16;
      crc = crc7(crc, c);
      TxRxbyte( c );

      c = arg >> 8;
      crc = crc7(crc, c);
      TxRxbyte( c );

      crc = crc7(crc, arg);
      TxRxbyte( arg );

      c = (crc << 1) | 1;    // CRC7 is sent in upper 7 bits of last byte
      TxRxbyte( c );
    }
  else
    {
      TxRxbyte( command | 0x40 );
      TxRxbyte( arg >> 24);
      TxRxbyte( arg >> 16);
      TxRxbyte( arg >> 8);
      TxRxbyte( arg );
      TxRxbyte( 0x01 );
    }

  /* Now wait for the response */

  for(i = 0; i < 64; i++ )
    {
      if( (response = TxRxbyte(0xff)) != 0xff )
	break;
    }
  return response;
}

int SetBlockLength(int length)
{
  int response;
  ASSERT_CS();
  response = SendCmd(CMD16, length, 0);
  NEGATE_CS();
  TxRxbyte(0xff);
  return response;
}

/*
  Initialize the MMC/SD/SDHC card. This includes setup of the SPI port.

  Based on Figure 7-2 in the  Physical Layer Simplified Specification Version 2.0

  SDHC cards not currently supported.

  return 0 on success and set the CardType
  non-zero return values indicate an error occured

*/

int SD_Init()
{
  int i, response;

  /* Start by initializing the SPI port */

  SPI_Init();

  /* Send dummy bytes with CS high to generate the required startup clock cycles */
  NEGATE_CS();                     
  for(i = 0; i < 50; i++)
    TxRxbyte(0xff);
  
  /* Send command 0  with CS asserted to switch to SPI mode. */

  ASSERT_CS();
  for(i = 0; i < 1000; i++)
    {
      response = SendCmd(CMD0, 0, 1);
      if(response & 1)
	break;
    }

  if( (response & 1) == 0 )         // No response to CMD0
    {
      NEGATE_CS();                       /* Raise chip select */
      return -1;                      /* an error occured */
    }

  /* For SD cards send the required CMD8, for some strange reason this
    command ignores the default condition of CRC checks disabled.

    I have one card (128MB Kingston micro-SD) that returns a fixed pattern
    of 0xaa. It also ignores all CRC7's.
    */

  response = SendCmd(CMD8, 0x1aa, 1);
  /*
    V1.X SD cards and MMC cards will not understand this command and will repond
    with the illegal command bit set.
  */
  if( response & 0x4 )
    {

      for( i = 1024; i > 0; i--)
	{
	  response = SendCmd(ACMD41, 0, 0);

	  if(response & 4)         
	    break;                 // If response indicates illegal command, quit	
	  if((response & 1) == 0)
	    break;              // Repeat if response indicates still in idle state

	}
      if( response & 4)           // Was not an SD card. Try MMC
	{
	  /* Send Command 1 to start the initialization process. */

	  for( i = 255; i > 0; i--)
	    {
	      response = SendCmd(CMD1, 0, 0);
	      if(response & 1)
		break;
	    }
	  NEGATE_CS();                       /* Raise chip select */
	  // If response is not zero, card did not come out of idle mode
	  if(response&1)
	    return -2;
	  CardType = MMC;

	  MAX_SPI_CLOCK();                // Increase SPI clock speed
          TxRxbyte(0xff);
	  SetBlockLength(512);
	  return 0;
	}
      if(response & 1)          // SD V1.x card initialized
	{

	  NEGATE_CS();                       /* Raise chip select */
	  CardType = SD;
	  MAX_SPI_CLOCK();                // Increase SPI clock speed
	  SetBlockLength(512);
	  return 0;
	}
    }
  else
    {
      /* CMD8 responds with 5 bytes, get the rest of them */
      TxRxbyte(0xff);
      TxRxbyte(0xff);
      response = TxRxbyte(0xff); 

      if(response != 1)
	{
	  NEGATE_CS();
	  return -3;
	}
      response = TxRxbyte(0xff);   // Check last byte which should be echoed

      if(response != 0xaa)
	{
	  NEGATE_CS();
	  return -4;
	}
    }

  /* Now send ACMD41 to start the initialization process. The HCS bit is cleared
     so SDHC cards will not initialize. 
  */

  for( i = 1024; i > 0; i--)
    {
      response = SendCmd(ACMD41, 0, 0);

      if((response & 1) == 0)
	break;              // Repeat if response indicates still in idle state
    }
  if(response & 1)
  {
    NEGATE_CS();
    return -5;
  }

/* CMD58 has a R3 response which is the normal R1 plus 4 more bytes
   which contain the OCR register.
*/
  busyWait();
  response = SendCmd(CMD58, 0, 0);  // Check CCS bit

  if((response = TxRxbyte(0xff)) & 0x40)
    CardType = SDHC;
  else
    CardType = SD;

  TxRxbyte(0xff);
  TxRxbyte(0xff);
  TxRxbyte(0xff);

  NEGATE_CS();                       /* Raise chip select */
  TxRxbyte(0xff);                   /* some extra clocks to keep it happy */

  MAX_SPI_CLOCK();                // Increase SPI clock speed
  SetBlockLength(512);
  return 0;

}
/*
  Read the Card Specific Data (CSD registers)
 */

#define CSD_SIZE     16

int readCSD(unsigned char * buf)
{
  int i, response;

 
  ASSERT_CS();                      /* Lower (assert) chip select */

  busyWait();

  response = SendCmd( CMD9, 0, 0);
  if( response )
    return -1;

  /* Now wait for the data token that indicates the start of the data */

  for( i = 255; i > 0; i--)
      if( TxRxbyte(0xff) == 0xfe )
	break;
  if( i <= 0 )
  {
    NEGATE_CS();
    return -1;          // read error
  }
  /* OK, card has been given the read command and address, all we have
   * to do now is read in the data.
   */

  for( i = 0; i < CSD_SIZE; i++ )
      *buf++ = TxRxbyte(0xff);  // get one byte

  // Read and discard the two CRC bytes
  TxRxbyte(0xff);
  TxRxbyte(0xff);

  NEGATE_CS();                       /* Raise chip select */
  TxRxbyte(0xff);                   /* some extra clocks to keep it happy */

  return 0;
}

/*
 * Read a sector
 */

int readAbsoluteSector( unsigned long sector, char *buffer)
{
  int i;
 
  ASSERT_CS();                      /* Lower (assert) chip select */
  busyWait();
  if(SendCmd(CMD17, sector << 9, 0))
    return -1;

  /* Now wait for the data token that indicates the start of the data */

  for( i = 1023; i > 0; i--)
      if( TxRxbyte(0xff) == 0xfe )
	break;
  if( i <= 0 )
  {
#ifdef DEBUG
printf("no read data token for sector %d\n", sector);
#endif
    NEGATE_CS();
    return -2;          // read error
  }
  /* OK, card has been given the read command and address, all we have
   * to do now is read in the data.
   */

  for( i = 0; i < SECTOR_SIZE; i++ )
      *buffer++ = TxRxbyte(0xff);  

  // Read and discard the two CRC bytes
  TxRxbyte(0xff);
  TxRxbyte(0xff);

  NEGATE_CS();                       /* Raise chip select */
  TxRxbyte(0xff);                   /* some extra clocks to keep it happy */

  return 0;
}




/*
  writeAbsoluteSector()

*/

int writeAbsoluteSector(unsigned long sector, unsigned char *buf)
{
  int c, i;

  /* First make sure that the MMC card even exists before proceeding. */

  if( CardType == NONE )
    return -1;

  ASSERT_CS();                      /* Lower (assert) chip select */

  busyWait();  

  if( SendCmd(CMD24, sector << 9, 0) != 0 )
    {
      /* An error occured. There is no good way to handle this
       * gracefully. 
       */
      return -1;
    }
  TxRxbyte(0xfe);                 /* Send the start block token followed by data */
  TxSector(buf, SECTOR_SIZE);

  TxRxbyte(0xff);        // Send a dummy 16 bit CRC which is ignored
  TxRxbyte(0xff);      

  /*
   * Now get the data response token.
   */

  for(i = 63; i >0; i--)
    {
      c = TxRxbyte(0xff) & 0x1f;
      if( c == 0x05 )     // Data stored correctly
	{
	  NEGATE_CS();    // Negate chip select */
	  TxRxbyte( 0xff );
	  return 0;
	}
      else if( c == 0x0b || c == 0x0d ) // Write error
	break;
    }
  NEGATE_CS();    // Negate chip select */
  TxRxbyte( 0xff );
  return -1;
}


/*
  StartMultiBlockWrite()

  Send the command to start a multiple block write.

*/


int StartMultiBlockWrite(unsigned long sector, int num)
{
  /* First make sure that the card even exists before proceeding. */

  if( CardType == NONE )
    return -1;;

  ASSERT_CS();                      /* Lower (assert) chip select */
  busyWait();

  /* Set the number of blocks to be written for pre-erase */
  if(CardType != MMC)
    SendCmd(ACMD23, num, 0);

  if( SendCmd(CMD25, sector << 9, 0) != 0 )
    {
      /* An error occured. There is no good way to handle this
       * gracefully. 
       */
      NEGATE_CS();
      return -1;
    }
  TxRxbyte(0xff);
  return 0;
}
/*
   BlockWrite()

   Write one block of a multiple block write

*/
int BlockWrite(unsigned char * buf)
{
  int c, i;

  ASSERT_CS();                    // This should already be on...
  busyWait();

  /* Send the start block token followed by data */
  
  TxRxbyte(0xfc);
  TxSector(buf, SECTOR_SIZE);
   
  TxRxbyte(0xff);                // 16 bit CRC (ignored)
  TxRxbyte(0xff);      

  /*
   * Now get the data response token.
   */

  for(i = 63; i >0; i--)
    {
      c = TxRxbyte(0xff) & 0x1f;
      if( c == 0x05 )     // Data stored correctly
	{
	  TxRxbyte( 0xff );
	  return 0;
	}
      else if( c == 0x0b || c == 0x0d ) // Write error
        break;
    }
  busyWait();
  TxRxbyte( 0xfd );
  return -1;
}

/* StopMultiBlockWrite()

   Stop a multiple block write command.
*/

int StopMultiBlockWrite()
{
  int i;


  /* Send the "Stop Tran" token */

  busyWait();
  TxRxbyte(0xfd);
#if 0
  i = SendStatus();         /* Now get the status. */

  i = SendCmd(ACMD22, 0, 0);     /* Determine the number of well written blocks */

  while(i = TxRxbyte(0xff) != 0xff)
  {

  }
#endif
  NEGATE_CS();
  TxRxbyte(0xff);
  return i;
}

